#ifndef _QTCHESS_GUI_
#define _QTCHESS_GUI_

/*
** -- C Includes --
*/

/*
** -- C++ Includes --
*/

/*
** -- Local Includes --
*/

#include "qtchess.h"
#include "qtchess_gui_opengl.h"

/*
** -- Qt Includes --
*/

#include <qframe.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qlayout.h>
#include <qstring.h>
#include <qmenubar.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qtextedit.h>
#include <qpopupmenu.h>
#include <qstatusbar.h>
#include <qmainwindow.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qdatetimeedit.h>

/*
** -- Class(es) --
*/

class qtchess_help_dialog: public QDialog
{
  Q_OBJECT

 public:

  /*
  ** -- Members --
  */

  /*
  ** -- Methods --
  */

  void setup(void);
  qtchess_help_dialog(QWidget *);

 private:

  /*
  ** -- Members --
  */

  QTextEdit *text;
  QPushButton *ok;

  /*
  ** -- Methods --
  */

  /*
  ** -- Slots --
  */

 private slots:

  void ok_cb(void);
};

class qtchess_setup_dialog: public QDialog
{
  Q_OBJECT

 public:

  /*
  ** -- Members --
  */

  /*
  ** -- Methods --
  */

  void disconnectedState(void);
  QLineEdit *getHostField(void);
  QLineEdit *getPortField(void);
  QLineEdit *getRHostField(void);
  QLineEdit *getRPortField(void);
  qtchess_setup_dialog(QWidget *);

 private:

  /*
  ** -- Members --
  */

  QLabel *host_label;
  QLabel *port_label;
  QLabel *rhost_label;
  QLabel *rport_label;
  QLineEdit *host;
  QLineEdit *port;
  QLineEdit *rhost;
  QLineEdit *rport;
  QPushButton *ok;
  QPushButton *cancel;
  QPushButton *disconnect;

  /*
  ** -- Methods --
  */

  /*
  ** -- Slots --
  */

 private slots:

  void ok_cb(void);
  void close_cb(void);
  void disconnect_cb(void);
};

class qtchess_history_dialog: public QDialog
{
  Q_OBJECT

 public:

  /*
  ** -- Members --
  */

  /*
  ** -- Methods --
  */

  void add_move(const struct move_s, const int);
  void clear_method(void);
  qtchess_history_dialog(QWidget *);

 private:

  /*
  ** -- Members --
  */

  QTextEdit *text;
  QPushButton *ok;
  QPushButton *clear;

  /*
  ** -- Methods --
  */

  /*
  ** -- Slots --
  */

 private slots:

  void ok_cb(void);
  void clear_cb(void);
};

class qtchess_promote_dialog: public QDialog
{
  Q_OBJECT

 public:

  /*
  ** -- Members --
  */

  /*
  ** -- Methods --
  */

  void setup(void);
  qtchess_promote_dialog(QWidget *);

  QComboBox *getMenu(void)
    {
      return menu;
    }

 private:

  /*
  ** -- Members --
  */

  QComboBox *menu;
  QPushButton *ok;

  /*
  ** -- Methods --
  */

  /*
  ** -- Slots --
  */

 private slots:

  void ok_cb(void);
};

class qtchess_clocks_dialog: public QDialog
{
  Q_OBJECT

 public:

  /*
  ** -- Methods --
  */

  qtchess_clocks_dialog(QWidget *);
  void init(void);
  void stopTimers(const int);
  void startTimers(const int);

 private:

  /*
  ** -- Members --
  */

  QTimer playert;
  QTimer opponentt;
  QTimeEdit *player;
  QTimeEdit *opponent;

  /*
  ** -- Slots --
  */

 private slots:
  void ok_cb(void);
  void updatePlayer(void);
  void updateOpponent(void);
};

class qtchess_gui: public QMainWindow
{
  Q_OBJECT

 public:

  /*
  ** -- Members --
  */

  /*
  ** -- Methods --
  */

  qtchess_gui(void)
    {
      statusbar = NULL;
      help_dialog = NULL;
      setup_dialog = NULL;
      clocks_dialog = NULL;
      history_dialog = NULL;
      promote_dialog = NULL;
    }

  ~qtchess_gui()
    {
      if(help_dialog != NULL)
	delete help_dialog;

      if(setup_dialog != NULL)
	delete setup_dialog;

      if(clocks_dialog != NULL)
	delete clocks_dialog;

      if(history_dialog != NULL)
	delete history_dialog;

      if(promote_dialog != NULL)
	delete promote_dialog;

#ifdef _DEBUG_
      (void) fprintf(stderr, "~qtchess_gui()\n");
#endif
    }

  QStatusBar *getStatusBar(void)
    {
      return statusbar;
    }

  openglWid *getGLBoard(void)
    {
      return glboard;
    }

  int exec(void);
  void init(void);
  void display(void);
  void initClocks(void);
  void stopTimers(const int);
  void startTimers(const int);
  void showErrorMsg(const char *);
  void showGameOver(const int);
  void showDisconnect(void);
  void showNewGameInfo(void);
  void notifyConnection(const QString &);
  virtual void closeEvent(QCloseEvent *);
  qtchess_setup_dialog *getSetupDialog(void);
  qtchess_history_dialog *getHistoryDialog(void);
  qtchess_promote_dialog *getPromoteDialog(void);

 private:

  /*
  ** -- Members --
  */

  openglWid *glboard;
  QStatusBar *statusbar;
  qtchess_help_dialog *help_dialog;
  qtchess_setup_dialog *setup_dialog;
  qtchess_clocks_dialog *clocks_dialog;
  qtchess_history_dialog *history_dialog;
  qtchess_promote_dialog *promote_dialog;

  /*
  ** -- Methods --
  */

  /*
  ** -- Slots --
  */

 private slots:

  void help(void);
  void quit(void);
  void about(void);
  void setup(void);
  void history(void);
  void newGame(void);
  void showClocks(void);
};

#endif
