/*
** -- Qt Includes --
*/

/*
** -- Local Includes --
*/

#include "qtchess.h"

extern qtchess *chess;
extern qtchess_gui *gui;
extern qtchess_comm *comm;
extern QApplication *qapp;

void openglWid::paintGL(void)
{
  int i = 0, j = 0, x = 0, y = 0,
    I = 0, J = 0, X = 0, Y = 0, rc = 0, found = 0, size = block_size,
    game_over = FALSE;
  QFont font;
  struct move_s current_move;

  /*
  ** All the drawing happens here.
  */

  glLineWidth(2.5);
  glPointSize(2.5);
  glClear(GL_COLOR_BUFFER_BIT); // Clears the screen.

  /*
  ** First an empty board.
  */

  for(i = 0; i < NSQUARES; i++)
    for(j = 0; j < NSQUARES; j++)
      {
	if((i + j) % 2 != 0)
	  glColor3f(0.8, 0.7, 0.5);
	else
	  glColor3f(0.5, 0.5, 0.6);

	x = p + i * size;
	y = p + j * size;
	glRecti(x,
		y,
		p + (i + 1) * size,
		p + (j + 1) * size);

	if(mouse_pressed)
	  if(point_pressed.x >= (p + i * size) &&
	     point_pressed.x <= (p + (i + 1) * size) &&
	     point_pressed.y >= (p + j * size) &&
	     point_pressed.y <= (p + (j + 1) * size))
	    {
	      found = 1;
	      X = p + i * size;
	      Y = p + j * size;
	      I = i;
	      J = j;
	    }

	/*
	** Draw the pieces.
	*/

	if(!qtchess_validate::isEmpty(chess->board[i][j]))
	  if(qtchess_validate::isWhite(chess->board[i][j]))
	    glColor3f(0.96, 0.96, 0.86);
	  else
	    glColor3f(0.502, 0.0, 0.0);

	if(qtchess_validate::isBishop(chess->board[i][j]))
	  {
	    glBegin(GL_POLYGON);
	    glVertex2i(x + BISHOP_X_OFFSET,
		       y + BISHOP_Y_OFFSET);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH,
		       y + BISHOP_Y_OFFSET);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH / 2 + 6,
		       y + BISHOP_Y_OFFSET + BISHOP_HEIGHT - 20);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH / 2 - 6,
		       y + BISHOP_Y_OFFSET + BISHOP_HEIGHT - 20);
	    glEnd();
	    glBegin(GL_POLYGON);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH / 2 + 6,
		       y + BISHOP_Y_OFFSET + BISHOP_HEIGHT - 20);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH / 2 + 18,
		       y + BISHOP_Y_OFFSET + BISHOP_HEIGHT - 10);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH / 2,
		       y + BISHOP_Y_OFFSET + BISHOP_HEIGHT);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH / 2 - 18,
		       y + BISHOP_Y_OFFSET + BISHOP_HEIGHT - 10);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH / 2 - 6,
		       y + BISHOP_Y_OFFSET + BISHOP_HEIGHT - 20);
	    glEnd();
	    glColor3f(0.0, 0.0, 0.0);
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + BISHOP_X_OFFSET,
		       y + BISHOP_Y_OFFSET);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH,
		       y + BISHOP_Y_OFFSET);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH / 2 + 6,
		       y + BISHOP_Y_OFFSET + BISHOP_HEIGHT - 20);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH / 2 - 6,
		       y + BISHOP_Y_OFFSET + BISHOP_HEIGHT - 20);
	    glEnd();
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH / 2 + 6,
		       y + BISHOP_Y_OFFSET + BISHOP_HEIGHT - 20);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH / 2 + 18,
		       y + BISHOP_Y_OFFSET + BISHOP_HEIGHT - 10);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH / 2,
		       y + BISHOP_Y_OFFSET + BISHOP_HEIGHT);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH / 2 - 18,
		       y + BISHOP_Y_OFFSET + BISHOP_HEIGHT - 10);
	    glVertex2i(x + BISHOP_X_OFFSET + BISHOP_WIDTH / 2 - 6,
		       y + BISHOP_Y_OFFSET + BISHOP_HEIGHT - 20);
	    glEnd();
	  }
	else if(qtchess_validate::isKing(chess->board[i][j]))
	  {
	    glRecti(x + KING_X_OFFSET, y + KING_Y_OFFSET,
		    x + KING_X_OFFSET + KING_B_WIDTH,
		    y + KING_Y_OFFSET + KING_B_HEIGHT);
	    glRecti(x + KING_X_OFFSET + 6,
		    y + KING_Y_OFFSET + KING_B_HEIGHT,
		    x + KING_X_OFFSET + 6 + KING_BT_WIDTH,
		    y + KING_Y_OFFSET + KING_B_HEIGHT + KING_BT_HEIGHT);
	    glRecti(x + KING_X_OFFSET + 5 + 3,
		    y + KING_Y_OFFSET + KING_B_HEIGHT + KING_BT_HEIGHT,
		    x + KING_X_OFFSET + 5 + 3 + KING_WIDTH,
		    y + KING_Y_OFFSET + KING_B_HEIGHT + KING_BT_HEIGHT +
		    KING_HEIGHT);
	    glRecti(x + KING_X_OFFSET + 5 + 3 + 6,
		    y + KING_Y_OFFSET + KING_B_HEIGHT +
		    KING_BT_HEIGHT + KING_HEIGHT,
		    x + KING_X_OFFSET + 5 + 3 + 6 + KING_VCROSS_WIDTH,
		    y + KING_Y_OFFSET + KING_B_HEIGHT +
		    KING_BT_HEIGHT + KING_HEIGHT + KING_VCROSS_HEIGHT);
	    glRecti(x + KING_X_OFFSET + 5 + 3,
		    y + KING_Y_OFFSET + KING_B_HEIGHT +
		    KING_BT_HEIGHT + KING_HEIGHT + 5,
		    x + KING_X_OFFSET + 5 + 3 + KING_HCROSS_WIDTH,
		    y + KING_Y_OFFSET + KING_B_HEIGHT +
		    KING_BT_HEIGHT + KING_HEIGHT + KING_HCROSS_HEIGHT + 5);
	    glColor3f(0.0, 0.0, 0.0);
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + KING_X_OFFSET, y + KING_Y_OFFSET);
	    glVertex2i(x + KING_X_OFFSET + KING_B_WIDTH,
		       y + KING_Y_OFFSET);
	    glVertex2i(x + KING_X_OFFSET + KING_B_WIDTH,
		       y + KING_Y_OFFSET + KING_B_HEIGHT);
	    glVertex2i(x + KING_X_OFFSET,
		       y + KING_Y_OFFSET + KING_B_HEIGHT);
	    glEnd();
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + KING_X_OFFSET + 5,
		       y + KING_Y_OFFSET + KING_B_HEIGHT);
	    glVertex2i(x + KING_X_OFFSET + 5 +
		       KING_BT_WIDTH,
		       y + KING_Y_OFFSET + KING_B_HEIGHT);
	    glVertex2i(x + KING_X_OFFSET + 5 +
		       KING_BT_WIDTH,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT);
	    glVertex2i(x + KING_X_OFFSET + 5,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT);
	    glEnd();
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + KING_X_OFFSET + 5 + 3,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT);
	    glVertex2i(x + KING_X_OFFSET + 5 + 3 +
		       KING_WIDTH,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT);
	    glVertex2i(x + KING_X_OFFSET + 5 + 3 +
		       KING_WIDTH,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT + KING_HEIGHT);
	    glVertex2i(x + KING_X_OFFSET + 5 + 3,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT + KING_HEIGHT);
	    glEnd();
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + KING_X_OFFSET + KING_VCROSS_WIDTH + 6 + 5 + 3,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT + KING_HEIGHT);
	    glVertex2i(x + KING_X_OFFSET + KING_VCROSS_WIDTH + 6 + 5 + 3,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT + KING_HEIGHT + 5);
	    glVertex2i(x + KING_X_OFFSET + KING_VCROSS_WIDTH + 6 + 6 + 5 + 3,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT + KING_HEIGHT + 5);
	    glVertex2i(x + KING_X_OFFSET + KING_VCROSS_WIDTH + 6 + 6 + 5 + 3,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT + KING_HEIGHT + KING_HCROSS_HEIGHT + 5);
	    glVertex2i(x + KING_X_OFFSET + KING_VCROSS_WIDTH + 6 + 5 + 3,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT + KING_HEIGHT + 5 + KING_HCROSS_HEIGHT);
	    glVertex2i(x + KING_X_OFFSET + KING_VCROSS_WIDTH + 6 + 5 + 3,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT + KING_HEIGHT + 5 +
		       KING_HCROSS_HEIGHT + 5);
	    glVertex2i(x + KING_X_OFFSET + 6 + 5 + 3,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT + KING_HEIGHT + 5 +
		       KING_HCROSS_HEIGHT + 5);
	    glVertex2i(x + KING_X_OFFSET + 6 + 5 + 3,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT + KING_HEIGHT + 5 + KING_HCROSS_HEIGHT);
	    glVertex2i(x + KING_X_OFFSET + 5 + 3,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT + KING_HEIGHT + 5 + KING_HCROSS_HEIGHT);
	    glVertex2i(x + KING_X_OFFSET + 5 + 3,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT + KING_HEIGHT + 5);
	    glVertex2i(x + KING_X_OFFSET + 6 + 5 + 3,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT + KING_HEIGHT + 5);
	    glVertex2i(x + KING_X_OFFSET + 6 + 5 + 3,
		       y + KING_Y_OFFSET + KING_B_HEIGHT +
		       KING_BT_HEIGHT + KING_HEIGHT);
	    glEnd();
	  }
	else if(qtchess_validate::isQueen(chess->board[i][j]))
	  {
	    glRecti(x + QUEEN_X_OFFSET, y + QUEEN_Y_OFFSET,
		    x + QUEEN_X_OFFSET + QUEEN_B_WIDTH,
		    y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT);
	    glRecti(x + QUEEN_X_OFFSET + 6,
		    y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT,
		    x + QUEEN_X_OFFSET + 6 + QUEEN_BT_WIDTH,
		    y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT + QUEEN_BT_HEIGHT);
	    glRecti(x + QUEEN_X_OFFSET + 5 + 3,
		    y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT + QUEEN_BT_HEIGHT,
		    x + QUEEN_X_OFFSET + 5 + 3 + QUEEN_WIDTH,
		    y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT + QUEEN_BT_HEIGHT +
		    QUEEN_HEIGHT);
	    glBegin(GL_POLYGON);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH / 2,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH + 10,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT + 20);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH / 2,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT + 5);
	    glEnd();
	    glBegin(GL_POLYGON);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH / 2,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH / 2,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT + 5);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 - 10,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT + 20);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT);
	    glEnd();
	    glBegin(GL_POLYGON);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH / 2,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT + 5);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       3 * QUEEN_WIDTH / 4,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT + 7);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH / 2,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT + 20);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH / 4,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT + 7);
	    glEnd();
	    glColor3f(0.0, 0.0, 0.0);
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + QUEEN_X_OFFSET, y + QUEEN_Y_OFFSET);
	    glVertex2i(x + QUEEN_X_OFFSET + QUEEN_B_WIDTH,
		       y + QUEEN_Y_OFFSET);
	    glVertex2i(x + QUEEN_X_OFFSET + QUEEN_B_WIDTH,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT);
	    glVertex2i(x + QUEEN_X_OFFSET,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT);
	    glEnd();
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + QUEEN_X_OFFSET + 5,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 +
		       QUEEN_BT_WIDTH,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 +
		       QUEEN_BT_WIDTH,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT);
	    glVertex2i(x + QUEEN_X_OFFSET + 5,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT);
	    glEnd();
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT);
	    glEnd();
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH + 10,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT + 20);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       3 * QUEEN_WIDTH / 4,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT + 8);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH / 2,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT + 20);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 +
		       QUEEN_WIDTH / 4,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT + 8);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3 - 10,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT + 20);
	    glVertex2i(x + QUEEN_X_OFFSET + 5 + 3,
		       y + QUEEN_Y_OFFSET + QUEEN_B_HEIGHT +
		       QUEEN_BT_HEIGHT + QUEEN_HEIGHT);
	    glEnd();
	  }
	else if(qtchess_validate::isKnight(chess->board[i][j]))
	  {
	    glBegin(GL_POLYGON);
	    glVertex2i(x + KNIGHT_X_OFFSET, y + KNIGHT_Y_OFFSET);
	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH + 2,
		       y + KNIGHT_Y_OFFSET);
	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH / 2,
		       y + KNIGHT_Y_OFFSET + KNIGHT_HEIGHT - 23);
 	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH / 2 + 1,
 		       y + KNIGHT_Y_OFFSET + KNIGHT_HEIGHT - 5);
	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH / 2 - 4,
		       y + KNIGHT_Y_OFFSET + KNIGHT_HEIGHT);
	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH / 2 - 4,
		       y + KNIGHT_Y_OFFSET + KNIGHT_HEIGHT);
	    glVertex2i(x + KNIGHT_X_OFFSET, y + KNIGHT_Y_OFFSET + 42);
	    glEnd();
	    glBegin(GL_POLYGON);
	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH / 2,
		       y + KNIGHT_Y_OFFSET + KNIGHT_HEIGHT - 23);
	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH + 5,
		       y + KNIGHT_Y_OFFSET + 15);
 	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH + 5,
 		       y + KNIGHT_Y_OFFSET + 25);
 	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH / 2 + 15,
 		       y + KNIGHT_Y_OFFSET + KNIGHT_HEIGHT - 10);
 	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH / 2 + 1,
 		       y + KNIGHT_Y_OFFSET + KNIGHT_HEIGHT - 5);
	    glEnd();
	    glColor3f(0.0, 0.0, 0.0);
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + KNIGHT_X_OFFSET, y + KNIGHT_Y_OFFSET);
	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH + 2,
		       y + KNIGHT_Y_OFFSET);
	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH / 2,
		       y + KNIGHT_Y_OFFSET + KNIGHT_HEIGHT - 23);
	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH + 5,
		       y + KNIGHT_Y_OFFSET + 15);
 	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH + 5,
 		       y + KNIGHT_Y_OFFSET + 25);
 	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH / 2 + 15,
 		       y + KNIGHT_Y_OFFSET + KNIGHT_HEIGHT - 10);
 	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH / 2 + 1,
 		       y + KNIGHT_Y_OFFSET + KNIGHT_HEIGHT - 5);
	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH / 2 - 4,
		       y + KNIGHT_Y_OFFSET + KNIGHT_HEIGHT);
	    glVertex2i(x + KNIGHT_X_OFFSET + KNIGHT_WIDTH / 2 - 4,
		       y + KNIGHT_Y_OFFSET + KNIGHT_HEIGHT);
	    glVertex2i(x + KNIGHT_X_OFFSET, y + KNIGHT_Y_OFFSET + 42);
	    glEnd();
	  }
	else if(qtchess_validate::isRook(chess->board[i][j]))
	  {
	    glRecti(x + ROOK_X_OFFSET,
		    y + ROOK_Y_OFFSET,
		    x + ROOK_X_OFFSET + ROOK_B_WIDTH,
		    y + ROOK_Y_OFFSET + ROOK_B_HEIGHT);
	    glRecti(x + ROOK_X_OFFSET + (ROOK_B_WIDTH - ROOK_WIDTH) / 2,
		    y + ROOK_Y_OFFSET + ROOK_B_HEIGHT,
		    x + ROOK_X_OFFSET + (ROOK_B_WIDTH - ROOK_WIDTH) / 2 +
		    ROOK_WIDTH,
		    y + ROOK_Y_OFFSET + ROOK_B_HEIGHT + ROOK_HEIGHT);
	    glRecti(x + ROOK_X_OFFSET + 5,
		    y + ROOK_Y_OFFSET + ROOK_B_HEIGHT + ROOK_HEIGHT,
		    x + ROOK_X_OFFSET + 5 + ROOK_T_WIDTH,
		    y + ROOK_Y_OFFSET + ROOK_B_HEIGHT + ROOK_HEIGHT +
		    ROOK_T_HEIGHT);
	    glColor3f(0.0, 0.0, 0.0);
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + ROOK_X_OFFSET, y + ROOK_Y_OFFSET);
	    glVertex2i(x + ROOK_X_OFFSET + ROOK_B_WIDTH, y + ROOK_Y_OFFSET);
	    glVertex2i(x + ROOK_X_OFFSET + ROOK_B_WIDTH,
		       y + ROOK_Y_OFFSET + ROOK_B_HEIGHT);
	    glVertex2i(x + ROOK_X_OFFSET,
		       y + ROOK_Y_OFFSET + ROOK_B_HEIGHT);
	    glEnd();
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + ROOK_X_OFFSET + 10 + ROOK_WIDTH,
		       y + ROOK_Y_OFFSET + ROOK_B_HEIGHT);
	    glVertex2i(x + ROOK_X_OFFSET + 10 + ROOK_WIDTH,
		       y + ROOK_Y_OFFSET + ROOK_B_HEIGHT + ROOK_HEIGHT);
	    glVertex2i(x + ROOK_X_OFFSET + 10,
		       y + ROOK_Y_OFFSET + ROOK_B_HEIGHT + ROOK_HEIGHT);
	    glVertex2i(x + ROOK_X_OFFSET + 10,
		       y + ROOK_Y_OFFSET + ROOK_B_HEIGHT);
	    glEnd();
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + ROOK_X_OFFSET + 5,
		       y + ROOK_Y_OFFSET + ROOK_B_HEIGHT + ROOK_HEIGHT);
	    glVertex2i(x + ROOK_X_OFFSET + 5 + ROOK_T_WIDTH,
		       y + ROOK_Y_OFFSET + ROOK_B_HEIGHT + ROOK_HEIGHT);
	    glVertex2i(x + ROOK_X_OFFSET + 5 + ROOK_T_WIDTH,
		       y + ROOK_Y_OFFSET + ROOK_B_HEIGHT + ROOK_HEIGHT +
		       ROOK_T_HEIGHT);
	    glVertex2i(x + ROOK_X_OFFSET + 5,
		       y + ROOK_Y_OFFSET + ROOK_B_HEIGHT + ROOK_HEIGHT +
		       ROOK_T_HEIGHT);
	    glEnd();
	  }
	else if(qtchess_validate::isPawn(chess->board[i][j]))
	  {
	    glRecti(x + PAWN_X_OFFSET,
		    y + PAWN_Y_OFFSET,
		    x + PAWN_X_OFFSET + PAWN_WIDTH,
		    y + PAWN_Y_OFFSET + PAWN_HEIGHT);
	    glBegin(GL_POLYGON);
	    glVertex2i(x + PAWN_X_OFFSET + PAWN_WIDTH,
		       y + PAWN_Y_OFFSET + PAWN_HEIGHT);
	    glVertex2i(x + PAWN_X_OFFSET + PAWN_WIDTH + 10,
		       y + PAWN_Y_OFFSET + PAWN_HEIGHT + 15);
	    glVertex2i(x + PAWN_X_OFFSET + PAWN_WIDTH / 2,
		       y + PAWN_Y_OFFSET + PAWN_HEIGHT + 30);
	    glVertex2i(x + PAWN_X_OFFSET - 10,
		       y + PAWN_Y_OFFSET + PAWN_HEIGHT + 15);
	    glVertex2i(x + PAWN_X_OFFSET, y + PAWN_Y_OFFSET + PAWN_HEIGHT);
	    glEnd();
	    glColor3f(0.0, 0.0, 0.0);
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + PAWN_X_OFFSET, y + PAWN_Y_OFFSET);
	    glVertex2i(x + PAWN_X_OFFSET + PAWN_WIDTH, y + PAWN_Y_OFFSET);
	    glVertex2i(x + PAWN_X_OFFSET + PAWN_WIDTH,
		       y + PAWN_Y_OFFSET + PAWN_HEIGHT);
	    glVertex2i(x + PAWN_X_OFFSET, y + PAWN_Y_OFFSET + PAWN_HEIGHT);
	    glEnd();
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(x + PAWN_X_OFFSET + PAWN_WIDTH,
		       y + PAWN_Y_OFFSET + PAWN_HEIGHT);
	    glVertex2i(x + PAWN_X_OFFSET + PAWN_WIDTH + 10,
		       y + PAWN_Y_OFFSET + PAWN_HEIGHT + 15);
	    glVertex2i(x + PAWN_X_OFFSET + PAWN_WIDTH / 2,
		       y + PAWN_Y_OFFSET + PAWN_HEIGHT + 30);
	    glVertex2i(x + PAWN_X_OFFSET - 10,
		       y + PAWN_Y_OFFSET + PAWN_HEIGHT + 15);
	    glVertex2i(x + PAWN_X_OFFSET, y + PAWN_Y_OFFSET + PAWN_HEIGHT);
	    glEnd();
	  }
      }

  /*
  ** Add the coordinate labels.
  */

  font.setBold(true);
  renderText(80, 40, QString("A"), font);
  renderText(80 + 80, 40, QString("B"), font);
  renderText(80 + 165, 40, QString("C"), font);
  renderText(80 + 245, 40, QString("D"), font);
  renderText(80 + 330, 40, QString("E"), font);
  renderText(80 + 410, 40, QString("F"), font);
  renderText(80 + 490, 40, QString("G"), font);
  renderText(80 + 575, 40, QString("H"), font);
  renderText(30, 90, QString("8"), font);
  renderText(30, 90 + 85, QString("7"), font);
  renderText(30, 90 + 170, QString("6"), font);
  renderText(30, 90 + 245, QString("5"), font);
  renderText(30, 90 + 335, QString("4"), font);
  renderText(30, 90 + 415, QString("3"), font);
  renderText(30, 90 + 495, QString("2"), font);
  renderText(30, 90 + 575, QString("1"), font);

  /*
  ** Now add some black borders to the board.
  */

  glColor3f(0.0, 0.0, 0.0);
  glBegin(GL_LINES);

  for(i = 0; i < NSQUARES + 1; i++)
    {
      /*
      ** Vertical lines.
      */

      glVertex2i(p + i * size, p);
      glVertex2i(p + i * size, p + 8 * size);

      /*
      ** Horizontal lines.
      */

      glVertex2i(p, p + i * size);
      glVertex2i(p + 8 * size, p + i * size);
    }

  glEnd();
  glFlush();

  /*
  ** Highlight the selected square.
  */

#ifdef _DEBUG_
  if(mouse_pressed && found)
#else
  if(mouse_pressed && found && chess->isReady())
#endif
    {
      /*
      ** Set the last square's border to its original color (black).
      */ 

      if(mouse_pressed == 1 && qtchess_validate::isEmpty(chess->board[I][J]))
	{
	  mouse_pressed = 0;
	  return;
	}
      else if(mouse_pressed == 1)
	{
	  if(chess->getMyColor() == WHITE &&
	     !qtchess_validate::isWhite(chess->board[I][J]))
	    {
	      mouse_pressed = 0;
	      glColor3f(1.0, 0.0, 0.0);
	    }
	  else if(chess->getMyColor() == BLACK &&
		  !qtchess_validate::isBlack(chess->board[I][J]))
	    {
	      mouse_pressed = 0;
	      glColor3f(1.0, 0.0, 0.0);
	    }
	  else
	    {
	      point_selected.x = I;
	      point_selected.y = J;
	      glColor3f(0.0, 1.0, 0.0);
	    }
	}
      else if(mouse_pressed == 2 && point_selected.x == I &&
	      point_selected.y == J)
	{
	  mouse_pressed = 1;
	  glColor3f(0.0, 1.0, 0.0);
	}
      else if(mouse_pressed == 2 &&
	      (rc = qtchess_validate::isValidMove(point_selected.y,
						  point_selected.x,
						  J, I,
						  chess->board
						  [point_selected.x]
						  [point_selected.y])))
	{
	  if(qtchess_validate::isKing(chess->board[I][J]))
	    game_over = TRUE;

	  mouse_pressed = 0;
	  glColor3f(0.0, 1.0, 0.0);
	  current_move.x1 = point_selected.y;
	  current_move.y1 = point_selected.x;
	  current_move.x2 = J;
	  current_move.y2 = I;
	  current_move.piece =
	    chess->board[point_selected.x][point_selected.y];
	  current_move.r_x1 = current_move.r_x2 = current_move.r_y1 =
	    current_move.r_y2 = -1;
	  current_move.promoted = 0;
	  current_move.pawn_2 = 0;
	  current_move.rook = -1;

	  /*
	  ** Piece promotion?
	  */

	  if(qtchess_validate::isPawn
	     (chess->board[point_selected.x][point_selected.y]) &&
	     (J == 0 || J == 7))
	    {
	      current_move.promoted = 1;
	      gui->getPromoteDialog()->setup();
#ifdef _DEBUG_
	      (void) fprintf
		(stderr, "currentItem() = %d\n",
		 gui->getPromoteDialog()->getMenu()->currentItem());
#endif

	      if(gui->getPromoteDialog()->getMenu()->currentItem() == 0)
		{
		  if(qtchess_validate::isBlack
		     (chess->board[point_selected.x][point_selected.y]))
		    chess->board[I][J] = current_move.piece = ROOK_BLACK;
		  else
		    chess->board[I][J] = current_move.piece = ROOK_WHITE;
		}
	      else if(gui->getPromoteDialog()->getMenu()->currentItem() == 1)
		{
		  if(qtchess_validate::isBlack
		     (chess->board[point_selected.x][point_selected.y]))
		    chess->board[I][J] = current_move.piece = QUEEN_BLACK;
		  else
		    chess->board[I][J] = current_move.piece = QUEEN_WHITE;
		}
	      else if(gui->getPromoteDialog()->getMenu()->currentItem() == 2)
		{
		  if(qtchess_validate::isBlack
		     (chess->board[point_selected.x][point_selected.y]))
		    chess->board[I][J] = current_move.piece = BISHOP_BLACK;
		  else
		    chess->board[I][J] = current_move.piece = BISHOP_WHITE;
		}
	      else
		{
		  if(qtchess_validate::isBlack
		     (chess->board[point_selected.x][point_selected.y]))
		    chess->board[I][J] = current_move.piece = KNIGHT_BLACK;
		  else
		    chess->board[I][J] = current_move.piece = KNIGHT_WHITE;
		}
	    }
	  else
	    chess->board[I][J] = chess->board[point_selected.x]
	      [point_selected.y];

	  if(rc == VALID_CASTLE)
	    {
#ifdef _DEBUG_
	      (void) fprintf(stderr, "Castling.\n");
	      (void) fprintf(stderr, "I = %d, J = %d\n", I, J);
#endif
	      if(I == 2 && J == 7) // Rook #1 Black
		{
		  chess->board[3][7] = ROOK1_BLACK;
		  chess->board[0][7] = EMPTY_SQUARE;
		  current_move.r_x1 = 7;
		  current_move.r_y1 = 3;
		  current_move.r_x2 = 7;
		  current_move.r_y2 = 0;
		  current_move.rook = ROOK1_BLACK;
		}
	      else if(I == 6 && J == 7) // Rook #2 Black
		{
		  chess->board[5][7] = ROOK2_BLACK;
		  chess->board[7][7] = EMPTY_SQUARE;
		  current_move.r_x1 = 7;
		  current_move.r_y1 = 5;
		  current_move.r_x2 = 7;
		  current_move.r_y2 = 7;
		  current_move.rook = ROOK2_BLACK;
		}
	      else if(I == 2 && J == 0) // Rook #1 White
		{
		  chess->board[3][0] = ROOK1_WHITE;
		  chess->board[0][0] = EMPTY_SQUARE;
		  current_move.r_x1 = 0;
		  current_move.r_y1 = 3;
		  current_move.r_x2 = 0;
		  current_move.r_y2 = 0;
		  current_move.rook = ROOK1_WHITE;
		}
	      else if(I == 6 && J == 0) // Rook #2 White
		{
		  chess->board[5][0] = ROOK2_WHITE;
		  chess->board[7][0] = EMPTY_SQUARE;
		  current_move.r_x1 = 0;
		  current_move.r_y1 = 5;
		  current_move.r_x2 = 0;
		  current_move.r_y2 = 7;
		  current_move.rook = ROOK2_WHITE;
		}

	      chess->board[point_selected.x][point_selected.y] = EMPTY_SQUARE;
	    }
	  else if(rc == VALID_PAWN_2)
	    current_move.pawn_2 = 1;
	  else if(rc == VALID_EN_PASSANT)
	    {
	      if(point_selected.y < J)
		chess->board[I][J - 1] = EMPTY_SQUARE;
	      else
		chess->board[I][J + 1] = EMPTY_SQUARE;
	    }

	  chess->board[point_selected.x][point_selected.y] = EMPTY_SQUARE;
	  update();
#ifdef Q_WS_WIN
	  updateGL();
#endif

	  for(i = 0; i < NSQUARES; i++)
	    for(j = 0; j < NSQUARES; j++)
	      current_move.board[i][j] = chess->board[i][j];

	  qapp->lock();
	  gui->getHistoryDialog()->add_move
	    (current_move,
	     qtchess_validate::isWhite(current_move.piece) ? WHITE : BLACK);
	  qapp->unlock();

	  /*
	  ** Send the move.
	  */

#ifdef _DEBUG_
	  for(i = 0; i < NSQUARES; i++)
	    for(j = 0; j < NSQUARES; j++)
	      (void) fprintf(stderr, "i = %d, j = %d, board[%d][%d] = %d\n",
			     i, j, i, j, chess->board[i][j]);
#endif

	  comm->sendMove(current_move);

	  if(game_over)
	    {
	      gui->stopTimers(PLAYER_TIMER);
	      gui->stopTimers(OPPONENT_TIMER);
	    }
	  else
	    {
	      gui->stopTimers(PLAYER_TIMER);
	      gui->startTimers(OPPONENT_TIMER);
	    }

	  return;
	}
      else
	{
	  mouse_pressed = 0;
	  glColor3f(1.0, 0.0, 0.0);
	}

      glLineWidth(2.5);
      glBegin(GL_LINES);

      /*
      ** Horizontal lines.
      */

      glVertex2i(X, Y);
      glVertex2i(X + block_size, Y);
      glVertex2i(X, Y + block_size);
      glVertex2i(X + block_size, Y + block_size);

      /*
      ** Vertical lines.
      */

      glVertex2i(X, Y);
      glVertex2i(X, Y + block_size);
      glVertex2i(X + block_size, Y);
      glVertex2i(X + block_size, Y + block_size);
      glEnd();
      glFlush();
    }
  else
    mouse_pressed = 0;

  glFlush();
}

void openglWid::initializeGL(void)
{
  /*
  ** Initiaze OpenGL "stuff".
  */

  /*
  ** Set a white background color.
  */

  glClearColor(1.0, 1.0, 1.0, 1.0);
  glShadeModel(GL_FLAT);
}

void openglWid::resizeGL(int w, int h)
{
  p = (w - 8 * block_size) / 2;
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0.0, w, 0.0, h);
  glMatrixMode(GL_MODELVIEW);
}

void openglWid::mousePressEvent(QMouseEvent *e)
{
  mouse_pressed += 1;
  point_pressed.x = e->x();
  point_pressed.y = OPEN_GL_DIMENSION - e->y();
  update();
}

openglWid::openglWid(QWidget *parent,
		     const char *name):QGLWidget(parent, name)
{
  mouse_pressed = 0;
  p = 8; // Reasonable default value.
  block_size = 82; // Good default value.
  point_pressed.x = point_pressed.y = -1;
}
